package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Activity;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author Dávid Želník - 469323
 */
@Repository
public class ActivityDaoImpl implements ActivityDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public Activity create(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        em.persist(activity);
        return activity;
    }

    @Override
    public Activity findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return em.find(Activity.class, id);
    }

    @Override
    public List<Activity> findByName(String name) {
        return em.createQuery("select a from Activity a where a.name = :name", Activity.class)
                .setParameter("name", name)
                .getResultList();
    }

    @Override
    public Activity update(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        if (activity.getName() == null) {
            throw new IllegalArgumentException("Name is null");
        }
        if (activity.getTime() == null) {
            throw new IllegalArgumentException("Time is null");
        }
        Activity fromDb = findById(activity.getId());
        if (fromDb != null)
            return em.merge(activity);

        throw new DataRetrievalFailureException("Not found in DB");
    }

    @Override
    public Activity delete(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        Activity found = findById(activity.getId());
        if(found == null) {
            throw new DataRetrievalFailureException("Activity not found");
        }
        em.remove(found);
        return activity;
    }

    @Override
    public List<Activity> findAll() {
        return em.createQuery("select a from Activity a", Activity.class)
                .getResultList();
    }

}
