package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Album;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author David Sapák
 */
@Repository
public class AlbumDaoImpl implements AlbumDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public Album create(Album album){
        if(album == null){
            throw new IllegalArgumentException("Album cannot be null");
        }
        em.persist(album); return album;
    }

    @Override
    public Album findById(String Id){
        if (Id == null)
            throw new IllegalArgumentException("Id si null");
        var result = em.find(Album.class, Id);
        if(result == null){
            throw new DataRetrievalFailureException("Nothing was found");
        }
        return result;
    }

    @Override
    public List<Album> findByName(String name){
        var result = em.createQuery("Select a from Album a where a.name = :name", Album.class)
                .setParameter("name", name)
                .getResultList();
        return result;
    }

    @Override
    public Album update(Album album){
        if(album == null)
            throw new IllegalArgumentException("Album is null.");
        if(album.getName() == null)
            throw new IllegalArgumentException("Album name is null.");
        if(album.getBand() == null)
            throw new IllegalArgumentException("Band in album is null.");
        Album fromDb = findById(album.getId());
        if (fromDb != null)
            return em.merge(album);
        throw new DataRetrievalFailureException("Not found in DB");
    }

    @Override
    public Album delete(Album album){
        if (album == null)
            throw new IllegalArgumentException("Album is null");
        if(album.getSongs() == null)
            throw new IllegalArgumentException("Album songs is null");
        if(album.getName() == null)
            throw new IllegalArgumentException("Album name is null");
        em.remove(findById(album.getId()));
        return album;
    }

    @Override
    public List<Album> findAll() {
        return em.createQuery("select a from Album a", Album.class)
                .getResultList();
    }
}
