package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Band;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author Dávid Želník - 469323
 */
@Repository
public class BandDaoImpl implements BandDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public Band create(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        em.persist(band);
        return band;
    }

    @Override
    public Band findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return em.find(Band.class, id);
    }

    @Override
    public List<Band> findByName(String name) {
        return em.createQuery("select b from Band b where b.name = :name", Band.class)
                .setParameter("name", name)
                .getResultList();
    }

    @Override
    public Band update(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        if (band.getBandMembers() == null) {
            throw new IllegalArgumentException("Members is null");
        }
        if (band.getName() == null) {
            throw new IllegalArgumentException("Name is null");
        }
        if (band.getType() == null) {
            throw new IllegalArgumentException("Type is null");
        }
        Band fromDb = findById(band.getId());
        if (fromDb != null)
            return em.merge(band);

        throw new DataRetrievalFailureException("Not found in DB");
    }

    @Override
    public Band delete(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Argument is null");
        }
        Band found = findById(band.getId());
        if(found == null) {
            throw new DataRetrievalFailureException("Band not found");
        }
        em.remove(found);
        return band;
    }

    @Override
    public List<Band> findAll() {
        return em.createQuery("select b from Band b", Band.class)
                .getResultList();
    }
}
