package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Manager;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author Pavel Vašina - 445606
 */
@Repository
public class ManagerDaoImpl implements ManagerDao {
    @PersistenceContext
    private EntityManager em;

    @Override
    public Manager create(Manager manager) {
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        em.persist(manager);
        return manager;
    }

    @Override
    public Manager findById(String id) {
        if(id == null){
            throw new IllegalArgumentException("Id is null");
        }
        return em.find(Manager.class, id);
    }

    @Override
    public List<Manager> findByName(String name) {
        if(name == null){
            throw new IllegalArgumentException("Name is null");
        }
        return em.createQuery("select m from Manager m where m.name = :name", Manager.class)
               .setParameter("name", name)
               .getResultList();
    }

    @Override
    public Manager update(Manager manager) {
       if(manager == null){
           throw new IllegalArgumentException("Manager is null");
       }
        if (manager.getName() == null) {
            throw new IllegalArgumentException("Name is null");
        }
       return em.merge(manager);
    }

    @Override
    public Manager delete(Manager manager) {
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        Manager found = findById(manager.getId());
        if(found == null) {
            throw new DataRetrievalFailureException("Manager not found");
        }
        em.remove(found);
        return manager;
    }

    @Override
    public List<Manager> findAll(){
        return em.createQuery("select m from Manager m", Manager.class)
                .getResultList();
    }
}
