package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Musician;

import java.util.List;

/**
 * Musician DAO interface
 *
 * @author Zuzana Tunegová
 */
public interface MusicianDao {

    /**
     * Creates a musician entity.
     *
     * @param musician Musician entity to create
     * @throws IllegalArgumentException if provided musician entity is null
     */
    void create(Musician musician);

    /**
     * Finds musician entity for given ID
     *
     * @param id Id of searched musician entity
     * @return null if musician entity is not found, musician entity otherwise
     * @throws IllegalArgumentException if provided ID is null
     */
    Musician findById(String id);

    /**
     * Finds list of musician entities for given name
     *
     * @param name Name of searched musician entity
     * @return Empty list if entities are not found, list of Musician entities otherwise
     */
    List<Musician> findByName(String name);

    /**
     * Updates a musician entity.
     *
     * @param musician Musician entity to update
     * @throws IllegalArgumentException if provided musician entity is null, or name is null, or type is null
     */
    Musician update(Musician musician);

    /**
     * Delete musician.
     *
     * @param musician Musician entity to be deleted
     * @throws IllegalArgumentException  if provided musician entity is null
     */
    void delete(Musician musician);

    /**
     * Finds all musician entities instances.
     *
     * @return all musician entities
     */
    List<Musician> findAll();

    /**
     * Finds all musicians without band.
     *
     * @return musician entities
     */
    List<Musician> findAvailableMusicians();

}
