package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Musician;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author Zuzana Tunegová
 */
@Repository
public class MusicianDaoImpl implements MusicianDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public void create(Musician musician) {
        if (musician == null)
            throw new IllegalArgumentException("Musician is null");
        em.persist(musician);
    }

    @Override
    public Musician findById(String id) {
        if (id == null)
            throw new IllegalArgumentException("Id si null");
        return em.find(Musician.class, id);
    }

    @Override
    public List<Musician> findByName(String name) {
        return em.createQuery("select m from Musician m where m.name = :name", Musician.class)
                .setParameter("name", name)
                .getResultList();
    }

    @Override
    public Musician update(Musician musician) {
        if (musician == null)
            throw new IllegalArgumentException("Musician is null");

        if (musician.getName() == null)
            throw new IllegalArgumentException("Null musician name");

        if (musician.getType() == null)
            throw new IllegalArgumentException("Null musician type");

        Musician fromDb = findById(musician.getId());
        if (fromDb != null)
            return em.merge(musician);

        throw new DataRetrievalFailureException("Musician not found");
    }

    @Override
    public void delete(Musician musician) {
        if (musician == null)
            throw new IllegalArgumentException("Musician is null");
        em.remove(findById(musician.getId()));
    }

    @Override
    public List<Musician> findAll() {
        return em.createQuery("select m from Musician m", Musician.class)
                .getResultList();
    }

    @Override
    public List<Musician> findAvailableMusicians() {
        return em.createQuery("select m from Musician m where m.hasBand =: hasBand", Musician.class)
                .setParameter("hasBand", false)
                .getResultList();
    }

}
