package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Offer;

import java.util.List;

public interface OfferDao {
    /**
     * Creates a offer entity.
     *
     * @param offer Offer entity to create
     * @throws IllegalArgumentException if provided offer entity is null
     */
    void create(Offer offer);

    /**
     * Finds offer entity for given ID
     *
     * @param id Id of searched offer entity
     * @return null if offer entity is not found, offer entity otherwise
     * @throws IllegalArgumentException if provided ID is null
     */
    Offer findById(String id);

    /**
     * Finds list of offer entities for given name
     *
     * @param name Name of searched offer entity
     * @return Empty list if entities are not found, list of Offer entities otherwise
     */
    List<Offer> findByName(String name);

    /**
     * Updates a offer entity.
     *
     * @param offer Offer entity to update
     * @throws IllegalArgumentException if provided offer entity is null, or name is null, or type is null
     */
    Offer update(Offer offer);

    /**
     * Delete offer.
     *
     * @param offer Offer entity to be deleted
     * @throws IllegalArgumentException  if provided offer entity is null
     */
    Offer delete(Offer offer);

    /**
     * Finds all offer entity instances.
     *
     * @return all offer entities
     */
    List<Offer> findAll();
}
