package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Offer;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

@Repository
public class OfferDaoImpl implements OfferDao{
    @PersistenceContext
    private EntityManager em;

    @Override
    public void create(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");
        if(offer.getBand() == null){
            throw new IllegalArgumentException("Band in offer is null");
        }
        em.persist(offer);
    }

    @Override
    public Offer findById(String id) {
        if (id == null)
            throw new IllegalArgumentException("Id si null");
        return em.find(Offer.class, id);
    }

    @Override
    public List<Offer> findByName(String name) {
        return em.createQuery("select m from Offer m where m.name = :name", Offer.class)
                .setParameter("name", name)
                .getResultList();
    }

    @Override
    public Offer update(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");

        if (offer.getName() == null)
            throw new IllegalArgumentException("Null offer name");

        if (offer.getBand() == null)
            throw new IllegalArgumentException("Null offer band");

        Offer fromDb = findById(offer.getId());
        if (fromDb != null)
            return em.merge(offer);

        throw new DataRetrievalFailureException("Offer not found");
    }

    @Override
    @Transactional
    public Offer delete(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");
        Offer found = findById(offer.getId());
        if(found == null) {
            throw new DataRetrievalFailureException("Offer not found");
        }
        em.remove(found);
        return found;
    }

    @Override
    public List<Offer> findAll() {
        return em.createQuery("select m from Offer m", Offer.class)
                .getResultList();
    }
}
