package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.entity.Song;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;

/**
 * @author David Sapák
 */
@Repository
public class SongDaoImpl implements SongDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public Song create(Song song) {
        if (song == null)
            throw new IllegalArgumentException("Song is null");
        em.persist(song);
        return song;
    }
    @Override
    public Song findById(String Id) {
        if (Id == null)
            throw new IllegalArgumentException("Id si null");
        Song result = em.find(Song.class, Id);
        if(result == null){
            throw new IllegalArgumentException("nothing was found.");
        }
        return result;
    }

    @Override
    public List<Song> findByName(String name) {
        List<Song> result = em.createQuery("select s from Song s where s.songName = :name", Song.class)
                .setParameter("name", name)
                .getResultList();
        return result;
    }

    @Override
    public Song update(Song song) {
        if(song == null)
            throw new IllegalArgumentException("Song is null.");
        if(song.getSongName() == null)
            throw new IllegalArgumentException("Song name is null.");
        if(song.getDuration() == null)
            throw new IllegalArgumentException("Song duration is null.");
        Song fromDb = findById(song.getId());
        if (fromDb != null)
            return em.merge(song);
        throw new DataRetrievalFailureException("Song not found");
    }

    @Override
    public Song delete(Song song) {
        if (song == null)
            throw new IllegalArgumentException("Song is null");
        Song found = findById(song.getId());
        if(found == null) {
            throw new DataRetrievalFailureException("Band not found");
        }
        em.remove(found);
        return found;
    }

    @Override
    public List<Song> findAll() {
        return em.createQuery("select s from Song s", Song.class)
                .getResultList();
    }
}
