package cz.fi.muni.pa165.dao;

import cz.fi.muni.pa165.entity.User;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.Optional;

@Repository
public class UserDaoImpl implements UserDao {

    @PersistenceContext
    private EntityManager em;

    @Override
    public void create(User user) {
        if (user == null)
            throw new IllegalArgumentException("User is null");
        em.persist(user);
    }

    @Override
    public User findById(String id) {
        if (id == null)
            throw new IllegalArgumentException("Id si null");
        return em.find(User.class, id);
    }

    @Override
    public List<User> findByName(String loginName) {
        return em.createQuery("select m from User m where m.name = :name", User.class)
                .setParameter("name", loginName)
                .getResultList();
    }

    @Override
    public User update(User user) {
        if (user == null)
            throw new IllegalArgumentException("User is null");

        if (user.getUserRole() == null)
            throw new IllegalArgumentException("Null user role");

        User fromDb = findById(user.getId());
        if (fromDb != null)
            return em.merge(user);

        throw new DataRetrievalFailureException("User not found");
    }

    @Override
    public void delete(User user) {
        if (user == null)
            throw new IllegalArgumentException("User is null");
        em.remove(findById(user.getId()));
    }

    @Override
    public List<User> findAll() {
        return em.createQuery("select m from User m", User.class)
                .getResultList();
    }

    public Optional<User> findByLoginName(String loginName) {
        if (loginName == null) throw new IllegalArgumentException("Argument is null");
        List<User> entity = em.createQuery("select entity from User entity where entity.name=:name", User.class)
                .setParameter("name", loginName)
                .getResultList(); // get list instead of single result to avoid exception catching if entity is not present
        if (entity.isEmpty()) return Optional.empty();
        else return Optional.of(entity.get(0));
    }
}

