package cz.fi.muni.pa165.entity;
import cz.fi.muni.pa165.enums.UserRole;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;


import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.UUID;

import static java.util.Objects.requireNonNull;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "UserDetails")
public class User  {

    @Id
    @NotNull
    private String id = UUID.randomUUID().toString();

    @NotBlank
    private String name;

    @NotBlank
    private String password;

    @OneToOne
    private Musician musician;

    @OneToOne
    private Manager manager;

    @NotNull
    private UserRole userRole;


    public User(@NotBlank String loginName, @NotBlank String password, @NotNull UserRole userRole) {
        this.name = loginName;
        this.password = password;
        this.userRole = userRole;
    }

    public User(@NotBlank String loginName, @NotBlank String password, @NotNull UserRole userRole, Musician musician) {
        this(loginName, password, userRole);
        this.musician = musician;
    }



    protected String toStringAdditionalInfo() {
        if (musician != null) {
            return String.format("login'%s', musician:'%s', systemRole: '%s'", getName(), musician.getName(), getUserRole().getRoleName());
        }

        if (manager != null){
            return String.format("login'%s', manager:%s, systemRole: '%s'", getName(), manager.getName(), getUserRole().getRoleName());
        }
        return null;
    }
}

