package cz.fi.muni.pa165.contollers;



import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.ActivityService;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.dto.activity.ActivityCreateDto;
import cz.fi.muni.pa165.dto.activity.ActivityDto;
import cz.fi.muni.pa165.dto.activity.ActivityUpdateDto;
import cz.fi.muni.pa165.entity.Activity;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/activity")
public class ActivityController {


    private ActivityService activityService;
    private BandService bandService;

    @Autowired
    public ActivityController(ActivityService activityService) {
        this.activityService = activityService;
    }

    @GetMapping
    public final List<ActivityDto> getActivities() {
        var result = activityService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }


    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final ActivityDto getActivity(@PathVariable("id") long id) throws ResourceNotFoundException {
        ActivityDto activityDto = DtoMapper.mapToDto(activityService.findById(String.valueOf(id)));
        return activityDto;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody ActivityCreateDto activityDto) throws ResourceAlreadyExistingException {
        try {
            Band band = bandService.findById(activityDto.getBandId());
            Activity activity = CreateDtoMapper.mapToEntity(activityDto);
            activity.setBand(band);
            activityService.create(activity);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update( @RequestBody ActivityUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Activity activity = activityService.findById(dto.getId());
        Activity updatedActivity = UpdateDtoMapper.mapToEntity(dto, activity);
        updatedActivity.setBand(band);
        activityService.update(updatedActivity);
    }


    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable("id") String id) {
        Activity activity = activityService.findById(id);
        activityService.delete(activity);
    }
}
