package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.MusicianService;
import cz.fi.muni.pa165.dto.band.BandCreateDto;
import cz.fi.muni.pa165.dto.band.BandDto;
import cz.fi.muni.pa165.dto.band.BandUpdateDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


import javax.validation.Valid;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/band")
public class BandController {
    private BandService bandService;
    private MusicianService musicianService;

    @Autowired
    public BandController(BandService bandService) {
        this.bandService = bandService;
    }

    @GetMapping
    public final List<BandDto> getBands() {
        var result = bandService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }


    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final BandDto getBand(@PathVariable("id") long id) throws ResourceNotFoundException {
        BandDto bandDto = DtoMapper.mapToDto(bandService.findById(String.valueOf(id)));
        return bandDto;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody BandCreateDto bandDto) throws ResourceAlreadyExistingException {
        try {
            Set<Musician> bandMembers = bandDto.getBandMembers().stream()
                    .map(id -> musicianService.findById(id))
                    .collect(Collectors.toSet());
            Band band = CreateDtoMapper.mapToEntity(bandDto);
            band.setBandMembers(bandMembers);
            bandService.create(band);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody BandUpdateDto dto) {
        Set<Musician> bandMembers = dto.getBandMembers().stream()
                .map(id -> musicianService.findById(id))
                .collect(Collectors.toSet());
        Band band = bandService.findById(dto.getId());
        Band updatedBand = UpdateDtoMapper.mapToEntity(dto, band);
        updatedBand.setBandMembers(bandMembers);
        bandService.update(updatedBand);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable String id) {
        Band band = bandService.findById(id);
        bandService.delete(band);
    }
}
