package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ManagerService;
import cz.fi.muni.pa165.dto.manager.ManagerCreateDto;
import cz.fi.muni.pa165.dto.manager.ManagerDto;
import cz.fi.muni.pa165.dto.manager.ManagerUpdateDto;
import cz.fi.muni.pa165.entity.Manager;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@CrossOrigin
@RestController
@RequestMapping("/manager")
public class ManagerController {

    private ManagerService managerService;

    @Autowired
    public ManagerController(ManagerService managerService) {
        this.managerService = managerService;
    }


    @GetMapping
    public final List<ManagerDto> getManagers() {
        return managerService
                .findAll()
                .stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toList());
    }


    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final ManagerDto getManager(@PathVariable("id") long id) throws ResourceNotFoundException {
        return DtoMapper.mapToDto(managerService.findById(String.valueOf(id)));
    }


    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody ManagerCreateDto managerCreateDto) throws ResourceAlreadyExistingException {
        try {
            managerService.create(CreateDtoMapper.mapToEntity(managerCreateDto));
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody ManagerUpdateDto dto) {
        Manager manager = managerService.findById(dto.getId());
        Manager updatedManager = UpdateDtoMapper.mapToEntity(dto, manager);
        managerService.update(updatedManager);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@PathVariable String id) {
        Manager manager = managerService.findById(id);
        managerService.delete(manager);
    }
}
