package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.OfferService;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.MusicianService;
import cz.fi.muni.pa165.dto.musician.MusicianCreateDto;
import cz.fi.muni.pa165.dto.musician.MusicianDto;
import cz.fi.muni.pa165.dto.musician.MusicianUpdateDto;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


@CrossOrigin
@RestController
@RequestMapping("/musicians")
public class MusicianController {

    private MusicianService musicianService;
    private BandService bandService;
    private OfferService offerService;

    @Autowired
    public MusicianController(MusicianService musicianService, BandService bandService, OfferService offerService) {
        this.musicianService = musicianService;
        this.bandService = bandService;
        this.offerService = offerService;
    }

    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final MusicianDto getMusician(@PathVariable("id") long id) throws ResourceNotFoundException {
        MusicianDto musicianDto = DtoMapper.mapToDto(musicianService.findById(String.valueOf(id)));
        return musicianDto;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody MusicianCreateDto musicianDto) throws ResourceAlreadyExistingException {
        try {
            Musician musician = CreateDtoMapper.mapToEntity(musicianDto);
            musicianService.create(musician);
            return;
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    @PostMapping(value = "/accept-offer/{id}", consumes = APPLICATION_JSON_VALUE)
    public final void acceptOffer(@RequestBody MusicianUpdateDto musicianDto, @PathVariable("id") String offerId){
        Musician musician = musicianService.findById(musicianDto.getId());
        Offer offer = offerService.findById(offerId);
        offer.getBand().addMusician(musician);
        musician.setHasBand(true);
        offerService.delete(offer);
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody MusicianUpdateDto dto) {
        Musician musician = musicianService.findById(dto.getId());
        Musician updatedMusician = UpdateDtoMapper.mapToEntity(dto, musician);
        musicianService.update(updatedMusician);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Musician musician = musicianService.findById(id);
        musicianService.delete(musician);
    }

    @GetMapping
    public final List<MusicianDto> getAvailableMusicians() {
        var result = musicianService.findAvailableMusicians().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }

}

