package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.BandService;
import cz.fi.muni.pa165.OfferService;
import cz.fi.muni.pa165.dto.offer.OfferCreateDto;
import cz.fi.muni.pa165.dto.offer.OfferDto;
import cz.fi.muni.pa165.dto.offer.OfferUpdateDto;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Offer;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

@CrossOrigin
@RestController
@RequestMapping("/offer")
public class OfferController {
    private OfferService offerService;
    private BandService bandService;

    @Autowired
    public OfferController(OfferService offerService) {
        this.offerService = offerService;
    }


    @GetMapping
    public final List<OfferDto> getOffers() {
        var result = offerService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }


    @GetMapping(value = "/{id}")
    public final OfferDto getOffer(@PathVariable("id") long id) throws ResourceNotFoundException {
        OfferDto offerDto = DtoMapper.mapToDto(offerService.findById(String.valueOf(id)));
        return offerDto;
    }


    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public final void create(@RequestBody OfferCreateDto offerDto) throws ResourceAlreadyExistingException {
        try {
            Band band = bandService.findById(offerDto.getBandId());
            Offer offer = CreateDtoMapper.mapToEntity(offerDto);
            offer.setBand(band);
            offerService.create(offer);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }


    @PutMapping
    public void update(@RequestBody OfferUpdateDto dto) {
        Band band = bandService.findById(dto.getBandId());
        Offer offer = offerService.findById(dto.getId());
        Offer updatedOffer = UpdateDtoMapper.mapToEntity(dto, offer);
        updatedOffer.setBand(band);
        offerService.update(updatedOffer);
    }


    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Offer offer = offerService.findById(id);
        offerService.delete(offer);
    }

}

