package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.AlbumService;
import cz.fi.muni.pa165.ApiUris;
import cz.fi.muni.pa165.SongService;
import cz.fi.muni.pa165.dto.song.SongCreateDto;
import cz.fi.muni.pa165.dto.song.SongDto;
import cz.fi.muni.pa165.dto.song.SongUpdateDto;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Song;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.inject.Inject;
import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


@CrossOrigin
@RestController
@RequestMapping("/song")
public class SongController {
    private SongService songService;
    private AlbumService albumService;

    @Autowired
    public SongController(SongService songService){
        this.songService = songService;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody SongCreateDto songDto) throws ResourceAlreadyExistingException {
        try {
            Album album = albumService.findById(songDto.getAlbumId());
            Song song = CreateDtoMapper.mapToEntity(songDto);
            album.addSong(song);
            songService.create(song);
        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }
    //find by id
    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final SongDto getSong(@PathVariable("id") long id) throws ResourceNotFoundException {
        return DtoMapper.mapToDto(songService.findById(String.valueOf(id)));
    }

    //find by name
    //TODO proper testing I am not sure this works
    @GetMapping(value = "/{name}", produces = APPLICATION_JSON_VALUE)
    public final List<SongDto> getSong(@PathVariable("name") String name) throws ResourceNotFoundException {
        return songService.findByName(name).stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
    }

    //update
    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody SongUpdateDto dto) {
        Album album = albumService.findById(dto.getAlbumId());
        Song song = songService.findById(dto.getId());
        album.addSong(song);
        Song updateSong = UpdateDtoMapper.mapToEntity(dto, song);
        songService.update(updateSong);
    }

    //delete
    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        Song Song = songService.findById(id);
        songService.delete(Song);
    }
    //find all
    @GetMapping
    public final List<SongDto> getSongs(){
        return songService.findAll().stream().map(DtoMapper:: mapToDto).collect(Collectors.toList());
    }


}
