package cz.fi.muni.pa165.contollers;

import cz.fi.muni.pa165.ManagerService;
import cz.fi.muni.pa165.MusicianService;
import cz.fi.muni.pa165.UserService;
import cz.fi.muni.pa165.dto.user.UserCreateDto;
import cz.fi.muni.pa165.dto.user.UserDto;
import cz.fi.muni.pa165.dto.user.UserUpdateDto;
import cz.fi.muni.pa165.entity.Manager;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.User;
import cz.fi.muni.pa165.exceptions.ResourceAlreadyExistingException;
import cz.fi.muni.pa165.exceptions.ResourceNotFoundException;
import cz.fi.muni.pa165.mapper.CreateDtoMapper;
import cz.fi.muni.pa165.mapper.DtoMapper;
import cz.fi.muni.pa165.mapper.UpdateDtoMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;


@CrossOrigin
@RestController
@RequestMapping("/user")
public class UserController {

    private UserService userService;
    private MusicianService musicianService;
    private ManagerService managerService;
    @Autowired
    public UserController(UserService userService) {
        this.userService = userService;
    }

    @GetMapping
    public final List<UserDto> getUsers() {
        var result = userService.findAll().stream().map(DtoMapper::mapToDto).collect(Collectors.toList());
        return result;
    }

    @GetMapping(value = "/{id}", produces = APPLICATION_JSON_VALUE)
    public final UserDto getUser(@PathVariable("id") long id) throws ResourceNotFoundException {
        UserDto userDto = DtoMapper.mapToDto(userService.findById(String.valueOf(id)));
        return userDto;
    }

    @GetMapping(value = "/{name}/{password}", produces = APPLICATION_JSON_VALUE)
    public final UserDto getUserAuth(@PathVariable("name") String name, @PathVariable("password") String password) throws ResourceNotFoundException {
        UserDto userDto = DtoMapper.mapToDto(userService.authenticated(name, password));
        return userDto;
    }

    @PostMapping(value = "/create", consumes = APPLICATION_JSON_VALUE)
    public final void create(@RequestBody UserCreateDto userDto) throws ResourceAlreadyExistingException {
        try {
            User user = CreateDtoMapper.mapToEntity(userDto);
            Musician musician;
            if (userDto.getMusicianId() != null){
                musician = musicianService.findById(userDto.getMusicianId());
                user.setMusician(musician);
            }

            Manager manager;
            if (userDto.getManagerId() != null){
                manager = managerService.findById(userDto.getManagerId());
                user.setManager(manager);
            }
            userService.create(user);

        } catch (Exception ex) {
            throw new ResourceAlreadyExistingException();
        }
    }

    @PutMapping(value = "/update", consumes = APPLICATION_JSON_VALUE)
    public void update(@Valid @RequestBody UserUpdateDto dto) {
        Musician musician = musicianService.findById(dto.getMusicianId());
        Manager manager = managerService.findById(dto.getManagerId());
        User user = userService.findById(dto.getId());
        User updatedUser = UpdateDtoMapper.mapToEntity(dto, user);
        if (musician != null)
            updatedUser.setMusician(musician);
        if (manager != null)
            updatedUser.setManager(manager);
        userService.update(updatedUser);
    }

    @DeleteMapping(value = "/{id}")
    public void delete(@RequestBody String id) {
        User user = userService.findById(id);
        userService.delete(user);
    }



}
