package cz.fi.muni.pa165.mapper;


import cz.fi.muni.pa165.dto.activity.ActivityDto;
import cz.fi.muni.pa165.dto.album.AlbumDto;
import cz.fi.muni.pa165.dto.band.BandDto;
import cz.fi.muni.pa165.dto.manager.ManagerDto;
import cz.fi.muni.pa165.dto.musician.MusicianDto;
import cz.fi.muni.pa165.dto.offer.OfferDto;
import cz.fi.muni.pa165.dto.song.SongDto;
import cz.fi.muni.pa165.dto.user.UserDto;
import cz.fi.muni.pa165.entity.*;
import cz.fi.muni.pa165.entity.Album;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Song;

import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

public class DtoMapper {

    public static MusicianDto mapToDto(Musician musician) {
        MusicianDto musicianDto =  new MusicianDto();
        musicianDto.setName(musician.getName());
        musicianDto.setId(musician.getId());
        musicianDto.setMusicianType(musician.getType().name());
        musicianDto.setHasBand(musician.isHasBand());
        return musicianDto;
    }


    public static BandDto mapToDto(Band band) {
        BandDto bandDto =  new BandDto();
        bandDto.setName(band.getName());
        bandDto.setBandType(band.getType().name());
        bandDto.setLogo(band.getLogo());
        bandDto.setId(band.getId());
        bandDto.setActivities(band.getActivities().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        Set<MusicianDto> set = new HashSet<>();
        for (Musician musician : band.getBandMembers()) {
            System.out.println(musician.getName());
            MusicianDto mapToDto = mapToDto(musician);
            set.add(mapToDto);
        }
        bandDto.setBandMembers(set);

        bandDto.setBandAlbums(band.getBandAlbums().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return bandDto;
    }

    public static AlbumDto mapToDto(Album album) {
        AlbumDto albumDto =  new AlbumDto();
        albumDto.setName(album.getName());
        albumDto.setBand(mapToDto(album.getBand()));
        albumDto.setId(album.getId());
        albumDto.setSongs(album.getSongs().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return albumDto;
    }

    public static ActivityDto mapToDto(Activity activity) {
        ActivityDto activityDto =  new ActivityDto();
        activityDto.setName(activity.getName());
        activityDto.setId(activity.getId());
        activityDto.setTime(activity.getTime());
        return activityDto;
    }

    public static SongDto mapToDto(Song song) {
        SongDto songDto =  new SongDto();
        songDto.setSongName(song.getSongName());
        songDto.setId(song.getId());
        songDto.setDuration(song.getDuration());
        return songDto;
    }

    public static ManagerDto mapToDto(Manager manager) {
        ManagerDto managerDto =  new ManagerDto();
        managerDto.setName(manager.getName());
        managerDto.setId(manager.getId());
        managerDto.setManagedBands(manager.getManagedBands().stream()
                .map(DtoMapper::mapToDto)
                .collect(Collectors.toSet()));
        return managerDto;
    }

    public static OfferDto mapToDto(Offer offer) {
        OfferDto offerDto =  new OfferDto();
        offerDto.setName(offer.getName());
        offerDto.setBand(mapToDto(offer.getBand()));
        offerDto.setId(offer.getId());
        return offerDto;
    }

    public static UserDto mapToDto(User user) {
        UserDto userDto = new UserDto();
        userDto.setId(user.getId());
        userDto.setName(user.getName());
        userDto.setPassword(user.getPassword());
        userDto.setUserRole(user.getUserRole().name());
        if (user.getMusician() != null)
            userDto.setMusician(DtoMapper.mapToDto(user.getMusician()));
        if (user.getManager() != null)
            userDto.setManager(DtoMapper.mapToDto(user.getManager()));
        return userDto;
    }

}
