package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.ActivityDao;
import cz.fi.muni.pa165.entity.Activity;
import org.springframework.stereotype.Service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Dávid Želník 469323
 */
@Service
@Transactional
public class ActivityServiceImpl implements ActivityService {

    private final ActivityDao activityRepository;

    @Autowired
    public ActivityServiceImpl(ActivityDao activityRepository) {
        this.activityRepository = activityRepository;
    }

    @Override
    public Activity create(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Activity is null");
        }
        return activityRepository.create(activity);
    }

    @Override
    public Activity findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("ID is null");
        }
        return activityRepository.findById(id);
    }

    @Override
    public List<Activity> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return activityRepository.findByName(name);
    }

    @Override
    public Activity update(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Activity is null");
        }

        Activity activityFromDB = activityRepository.findById(activity.getId());
        if (activityFromDB == null) {
            throw new IllegalArgumentException("Activity not found in database");
        }

        return activityRepository.update(activity);
    }

    @Override
    public Activity delete(Activity activity) {
        if (activity == null) {
            throw new IllegalArgumentException("Activity is null");
        }

        Activity activityFromDb = activityRepository.findById(activity.getId());
        if (activityFromDb == null) {
            throw new IllegalArgumentException("Activity not found in database");
        }

        return activityRepository.delete(activity);
    }

    @Override
    public List<Activity> findAll() {
        return activityRepository.findAll();
    }
}
