package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.AlbumDao;
import cz.fi.muni.pa165.entity.Album;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author David Sapak
 */
@Service
public class AlbumServiceImpl implements AlbumService {

    private final AlbumDao albumRepository;

    @Autowired
    public AlbumServiceImpl(AlbumDao albumRepository){
        this.albumRepository = albumRepository;
    }

    @Override
    public Album create(Album album){
        if(album == null){
            throw new IllegalArgumentException("album cannot be null");
        }
        return albumRepository.create(album);
    }

    @Override
    public Album findById(String id){
        if(id == null){
            throw new IllegalArgumentException("Id cannot be null");
        }
        return albumRepository.findById(id);
    }
    @Override
    public List<Album> findByName(String name) {
        return albumRepository.findByName(name);
    }
    @Override
    public Album update(Album album) {
        if(album == null){
            throw new IllegalArgumentException("album cannot be null");
        }
        if(album.getName() == null){
            throw new IllegalArgumentException("album name cannot be null");
        }
        if(album.getBand() == null){
            throw new IllegalArgumentException("Band cannot be null");
        }
        return albumRepository.update(album);
    }
    @Override
    public Album delete(Album album) {
        if(album==null){
            throw new IllegalArgumentException("Album cannot be null");
        }
        Album repo = albumRepository.findById(album.getId());
        if(repo == null){
            throw new IllegalArgumentException("Album has to in database");
        }

        return albumRepository.delete(album);
    }

    @Override
    public List<Album> findAll() {
        return albumRepository.findAll();
    }
}
