package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.BandDao;
import cz.fi.muni.pa165.entity.Band;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Dávid Želník 469323
 */
@Service
public class BandServiceImpl implements BandService {

    private final BandDao bandRepository;

    @Autowired
    public BandServiceImpl(BandDao bandRepository) {
        this.bandRepository = bandRepository;
    }

    @Override
    public Band create(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }
        return bandRepository.create(band);
    }

    @Override
    public Band findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return bandRepository.findById(id);
    }

    @Override
    public List<Band> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return bandRepository.findByName(name);
    }

    @Override
    public Band update(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }

        Band bandFromDB = bandRepository.findById(band.getId());
        if (bandFromDB == null) {
            throw new IllegalArgumentException("Band not found in database");
        }

        return bandRepository.update(band);
    }

    @Override
    public Band delete(Band band) {
        if (band == null) {
            throw new IllegalArgumentException("Band is null");
        }

        Band bandFromDB = bandRepository.findById(band.getId());
        if (bandFromDB == null) {
            throw new IllegalArgumentException("Band not found in database");
        }

        return bandRepository.delete(band);
    }

    @Override
    public List<Band> findAll() {
        return bandRepository.findAll();
    }
}
