package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.ManagerDao;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Manager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;

/**
 * @author Pavel Vašina - 445606
 */
@Service
public class ManagerServiceImpl implements ManagerService {
    private final ManagerDao managerRepository;

    @Autowired
    public ManagerServiceImpl(ManagerDao managerRepository){
        this.managerRepository = managerRepository;
    }

    @Override
    public Manager create(Manager manager) {
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        return managerRepository.create(manager);
    }

    @Override
    public Manager findById(String id) {
        if(id == null){
            throw new IllegalArgumentException("Id is null");
        }
        return managerRepository.findById(id);
    }

    @Override
    public List<Manager> findByName(String name) {
        if(name == null){
            throw new IllegalArgumentException("Name is null");
        }
        return managerRepository.findByName(name);
    }

    @Override
    public Manager update(Manager manager) {
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        return managerRepository.update(manager);
    }

    @Override
    public Manager delete(Manager manager) {
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        Manager managerFromDb = managerRepository.findById(manager.getId());
        if (managerFromDb == null) {
            throw new IllegalArgumentException("Manager not found in database");
        }
        return managerRepository.delete(manager);
    }

    @Override
    public Set<Band> addManagedBand (Band band, Manager manager){
        if(manager == null){
            throw new IllegalArgumentException("Manager is null");
        }
        if(band == null){
            throw new IllegalArgumentException("Band is null");
        }
        Set<Band> bands = manager.addManagedBand(band);
        Manager updatedManager = managerRepository.update(manager);
        if (updatedManager != null){
            return updatedManager.getManagedBands();
        }
        return bands;
    }

    @Override
    public List<Manager> findAll(){
        return managerRepository.findAll();
    }
}
