package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.MusicianDao;
import cz.fi.muni.pa165.entity.Band;
import cz.fi.muni.pa165.entity.Musician;
import cz.fi.muni.pa165.entity.Offer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class MusicianServiceImpl implements MusicianService{

    private final MusicianDao musicianRepository;

    @Autowired
    public MusicianServiceImpl(MusicianDao musicianRepository){
        this.musicianRepository = musicianRepository;
    }

    @Override
    @Transactional
    public void create(Musician musician) {
        if (musician == null) {
            throw new IllegalArgumentException("Musician is null");
        }
        musicianRepository.create(musician);
    }

    @Override
    @Transactional
    public Musician findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return musicianRepository.findById(id);
    }

    @Override
    @Transactional
    public List<Musician> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return musicianRepository.findByName(name);
    }

    @Override
    @Transactional
    public Musician update(Musician musician) {
        if (musician == null)
            throw new IllegalArgumentException("Musician is null");

        Musician musicianFromDB = musicianRepository.findById(musician.getId());
        if (musicianFromDB == null)
            throw new IllegalArgumentException("Musician not found in database");

        return musicianRepository.update(musician);
    }

    @Override
    @Transactional
    public void delete(Musician musician) {
        if (musician == null) {
            throw new IllegalArgumentException("Musician is null");
        }

        Musician musicianFromDB = musicianRepository.findById(musician.getId());
        if (musicianFromDB == null) {
            throw new IllegalArgumentException("Musician not found in database");
        }
        musicianRepository.delete(musician);
    }

    @Override
    @Transactional
    public List<Musician> findAll() {
        return musicianRepository.findAll();
    }

    @Override
    @Transactional
    public void acceptOffer(Offer offer, Musician musician){
        if(offer == null){
            throw new IllegalArgumentException("Offer cannot be null");
        }
        if(musician == null){
            throw new IllegalArgumentException("Musician cannot be null");
        }
        Band band = offer.getBand();
        musician.setHasBand(true);
        band.addMusician(musician);
    }

    @Override
    @Transactional
    public List<Musician> findAvailableMusicians(){
        List<Musician> musicians = musicianRepository.findAvailableMusicians();
        return musicians;
    }
}
