package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.OfferDao;
import cz.fi.muni.pa165.entity.Offer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class OfferServiceImpl implements OfferService{
    private final OfferDao offerRepository;

    @Autowired
    public OfferServiceImpl(OfferDao offerRepository){
        this.offerRepository = offerRepository;
    }

    @Override
    public void create(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");
        offerRepository.create(offer);
    }

    @Override
    public Offer findById(String id) {
        if (id == null)
            throw new IllegalArgumentException("ID is null");
        return offerRepository.findById(id);
    }

    @Override
    public List<Offer> findByName(String name) {
        if (name == null)
            throw new IllegalArgumentException("Name is null");
        return offerRepository.findByName(name);
    }

    @Override
    public Offer update(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");
        Offer offerFromDb = offerRepository.findById(offer.getId());
        if (offerFromDb == null)
            throw new IllegalArgumentException("Offer not found in database");
        return offerRepository.update(offer);
    }

    @Override
    public void delete(Offer offer) {
        if (offer == null)
            throw new IllegalArgumentException("Offer is null");
        Offer offerFroDb = offerRepository.findById(offer.getId());
        if (offerFroDb == null)
            throw new IllegalArgumentException("Offer not found in database");
        offerRepository.delete(offer);
    }

    @Override
    public List<Offer> findAll() {
        return offerRepository.findAll();
    }
}
