package cz.fi.muni.pa165;

import cz.fi.muni.pa165.entity.User;

import java.util.List;

/**
 * @author Zuzana Tunegová
 */
public interface UserService {

    /**
     * Creates a user entity.
     *
     * @param user User entity to create
     * @throws IllegalArgumentException if provided user entity is null
     */
    void create(User user);

    /**
     * Finds user entity for given ID
     *
     * @param id Id of searched user entity
     * @return null if user entity is not found, user entity otherwise
     * @throws IllegalArgumentException if provided ID is null
     */
    User findById(String id);

    /**
     * Finds list of user entities for given name
     *
     * @param name Name of searched user entity
     * @return Empty list if entities are not found, list of User entities otherwise
     */
    List<User> findByName(String name);

    /**
     * Updates a user entity.
     *
     * @param user User entity to update
     * @throws IllegalArgumentException if provided user entity is null, or name is null, or type is null
     */
    User update(User user);

    /**
     * Delete user.
     *
     * @param user User entity to be deleted
     * @throws IllegalArgumentException if provided user entity is null
     */
    void delete(User user);

    /**
     * Finds all user entity instances.
     *
     * @return all user entities
     */
    List<User> findAll();

    /**
     * Authenticates users
     *
     * @return user if exists, else null
     */
    User authenticated(String userName, String password);
}
 
