package cz.fi.muni.pa165;

import cz.fi.muni.pa165.dao.UserDao;
import cz.fi.muni.pa165.entity.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class UserServiceImpl implements UserService{

    private final UserDao userRepository;

    @Autowired
    public UserServiceImpl(UserDao userRepository){
        this.userRepository = userRepository;
    }

    @Override
    @Transactional
    public void create(User user) {
        user.setPassword(String.valueOf(user.getPassword().hashCode()));
        if (user == null) {
            throw new IllegalArgumentException("User is null");
        }
        userRepository.create(user);
    }

    @Override
    @Transactional
    public User findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Id is null");
        }
        return userRepository.findById(id);
    }

    @Override
    @Transactional
    public List<User> findByName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Name is null");
        }
        return userRepository.findByName(name);
    }

    @Override
    @Transactional
    public User update(User user) {
        if (user == null)
            throw new IllegalArgumentException("User is null");

        User userFromDB = userRepository.findById(user.getId());
        if (userFromDB == null)
            throw new IllegalArgumentException("User not found in database");

        return userRepository.update(user);
    }

    @Override
    @Transactional
    public void delete(User user) {
        if (user == null) {
            throw new IllegalArgumentException("User is null");
        }

        User userFromDB = userRepository.findById(user.getId());
        if (userFromDB == null) {
            throw new IllegalArgumentException("User not found in database");
        }
        userRepository.delete(user);
    }

    @Override
    @Transactional
    public List<User> findAll() {
        return userRepository.findAll();
    }


    @Override
    public User authenticated(String userName, String password){
        String pass = String.valueOf(password.hashCode());
        List<User> users = findByName(userName);
        if (users.isEmpty()){
            return null;
        }
        Optional<User> foundUser = users.stream().filter(user -> user.getPassword().equals(pass)).findFirst();
        if (foundUser.isEmpty()){
            return null;
        }
        return foundUser.get();
    }
}

